package ru.dbotthepony.mc.otm.container

import net.minecraft.world.entity.player.Player
import java.util.function.Predicate

class SlottedContainerBuilder {
	fun interface SlotProvider {
		fun create(container: SlottedContainer, index: Int): ContainerSlot
	}

	private val slots = ArrayList<SlotProvider>()
	private var stillValid = Predicate<Player> { true }
	private val globalChangeListeners = ArrayList<Runnable>()

	fun add(slot: SlotProvider): SlottedContainerBuilder {
		slots.add(slot)
		return this
	}

	fun add(amount: Int, provider: SlotProvider): SlottedContainerBuilder {
		for (i in 0 until amount)
			slots.add(provider)

		return this
	}

	fun stillValid(predicate: Predicate<Player>): SlottedContainerBuilder {
		this.stillValid = predicate
		return this
	}

	fun onChanged(listener: Runnable): SlottedContainerBuilder {
		globalChangeListeners.add(listener)
		return this
	}

	fun copy(): SlottedContainerBuilder {
		val copy = SlottedContainerBuilder()
		copy.slots.addAll(slots)
		copy.globalChangeListeners.addAll(globalChangeListeners)
		copy.stillValid = stillValid
		return copy
	}

	fun build(): SlottedContainer {
		return SlottedContainer(slots, stillValid, globalChangeListeners.toTypedArray())
	}
}
